% Example script for the Diffusion pathfinding algorithm 

% Diffusion pathfinding algorithm (DPA) - Version 1.0 
% Date: 02/03/2016
% Author: Benjamin Ivorra 
% MOMAT research group, IMI-Institute & Applied Mathematics Department. Complutense University of Madrid, Spain.
% Contact for support: ivorra@mat.ucm.es

% Help guide:

% DPA is a 2D pathfinding algorithm based on a diffusion technique starting from the target point up to the initial point of the path. 
% The environment is coded by using a rectangular grid. 

% Input variables:

% ndx   = 2D grid size in the X-axis - Natural number greater than 5
% ndy   = 2D grid size in the Y-axis - Natural number greater than 5
% xo    = initial position in the X-axis - Natural number in [2,ndx-1]
% yo    = initial position in the X-axis - Natural number in [2,ndx-1]
% xf    = final position in the X-axis - Natural number in [2,ndx-1] 
% yf    = final position in the Y-axis - Natural number in [2,ndx-1]
% wall  = an integer (ndx,ndy) matrix corresponding to the wall that path cannot
% cross: 0= no wall | 1= wall
% displ = define if graphical and textual outputs should be displayed on the
% screen - 1 = On | 0 = Off 

% Output variables:

% sol    = a matrix containing the solution path: 1st column= X coordinate | 2nd
% column = Y coordinate. If no solution was found this matrix is void.
% flag   = an integer returning: 0= No solution was found | 1= Solution found |2= Fatal error
% runt   = Running time of the DPA without output generation (in secs)
% perft  = Performance time of the DPA defined as: nodes in the wall matrix
% equal to 0/runt (in Nodes per secs).
% emess  = a text containing the fatal error message

clear all
close all

% Geometry grid size
ndx=100;
ndy=100;

% Initial position
xo=min(max(round(0.04*ndx),2),ndx);
yo=min(max(round(0.04*ndy),2),ndy);

% Target position
xf=min(max(round(0.3*ndx),2),ndx);
yf=min(max(round(0.04*ndy),2),ndy);

% Building walls
wall=ones(ndx,ndy);
wall(2:end-1,2:end-1)=zeros(ndx-2,ndy-2);
wall(round(ndx*0.10),round((25)/100*ndy:(90)/100*ndy))=1;
wall(round(15/100*ndx),round((30)/100*ndy:(100)/100*ndy))=1;
wall(round(70/100*ndx),round((5)/100*ndy:(45)/100*ndy))=1;
wall(round(50/100*ndx),round(1:(20)/100*ndy))=1;
wall(round(50/100*ndx),round((30)/100*ndy:(50)/100*ndy))=1;
wall(round(25/100*ndx),round(1:(95)/100*ndy))=1;
wall(round((5)/100*ndx:(95)/100*ndx),round(25/100*ndy))=1;
wall(round((40)/100*ndx:(100)/100*ndx),round(50/100*ndy))=1;
wall(round((25)/100*ndx:(90)/100*ndx),round(80/100*ndy))=1;

% Output display
displ=1;

[sol,flag,runt,perft,emess]=DPA(xo,yo,xf,yf,ndx,ndy,wall,displ)
